/*
 * Copyright (C) 2020 - 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <Imlib2.h>
#include <nil-cairo.h>
#include <stdlib.h>
#include <string.h>

//#define ENABLE_ILBM /* not working */
//#define ENABLE_WEBP /* not working */

/*

  argb    - simple netpbm like format
* bmp
  bz2     - reading compressed files?
  ff
* gif
* ico
  id3
* jpeg
* lbm
* png
* pnm
* tga
* tiff
* webp
* xpm
  zlib

*/

struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static const char **
_imlib2_get_mime_types()
{
    static const char *mime_types[] = {
        "image/bmp",
        "image/gif",
        "image/vnd.microsoft.icon",
        "image/jpeg",
#ifdef ENABLE_ILBM
        "image/x-ilbm",
#endif
        "image/png",
        "image/x-portable-anymap", "image/x-portable-bitmap", "image/x-portable-graymap", "image/x-portable-greymap", "image/x-portable-pixmap",
        "image/x-tga",
        "image/tiff",
#ifdef ENABLE_WEBP
        "image/webp",
#endif
        "image/x-xpixmap", "image/x-xpmi",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_imlib2_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    h->info = info;
    h->info->threadsafe_render = 1;
    h->surface = NULL;

    return h;
}


static void
_imlib2_free(abydos_plugin_handle_t *h)
{
    if(h->surface)
        cairo_surface_destroy(h->surface);
    free(h);
}


static cairo_surface_t *
_surface_from_context(int width, int height)
{
    nile_format_t format = imlib_image_has_alpha() ? NILE_FORMAT_b8g8r8a8 : NILE_FORMAT_b8g8r8;
    return nil_cairo_surface_create_from_direct((const uint8_t*)imlib_image_get_data_for_reading_only(),
                                                width,height,-1,
                                                4,format,
                                                0);
}


#if 0
static int
_imlib2_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
}
#endif


static int
_imlib2_create_from_file(abydos_plugin_handle_t *h,const char *filename)
{
    Imlib_Image image = imlib_load_image_immediately_without_cache(filename);
    if(!image)
        return -1;
    imlib_context_set_image(image);

    h->info->width = imlib_image_get_width();
    h->info->height = imlib_image_get_height();
    h->surface = _surface_from_context(h->info->width,h->info->height);

    imlib_free_image_and_decache();
    return 0;
}


static cairo_surface_t *
_imlib2_get_image_surface(abydos_plugin_handle_t *h,
                          int page,
                          cairo_bool_t *layer_visibility,
                          int variant,
                          double time,
                          int frame)
{
    return cairo_surface_reference(h->surface);
}


#if 0
static void
_imlib2_get_duration()
{
}
#endif


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _imlib2_get_mime_types,
        .new = _imlib2_new,
        .free = _imlib2_free,
        //.create_from_data = _imlib2_create_from_data,
        .create_from_file = _imlib2_create_from_file,
        .get_image_surface = _imlib2_get_image_surface
        //.get_duration = _imlib2_get_duration
    };
    return &p;
}
