#include <abydos-plugin.h>
#include <debug.h>
#include <stdlib.h>
#include <string.h>

#define J40_IMPLEMENTATION
#define J40_CONFIRM_THAT_THIS_IS_EXPERIMENTAL_AND_POTENTIALLY_UNSAFE
#define CHAR_BIT 8
#include "j40.h"


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
};


static const char **
_j40_get_mime_types()
{
    static const char *mime_types[] = {
        "image/jxl",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_j40_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    h->info = info;

    return h;
}


static void
_j40_free(abydos_plugin_handle_t *h)
{
    free(h);
}


static void
_freefunc()
{
    DEBUG_p("begin");
}


static int
_j40_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
    j40_image image;
    int res;

    j40_from_memory(&image, (void*)data, len, _freefunc);
    j40_output_format(&image, J40_RGBA, J40_U8X4);

    // JPEG XL supports animation, so `j40_next_frame` calls can be called multiple times
    if (j40_next_frame(&image)) {
        j40_frame frame = j40_current_frame(&image);
        j40_pixels_u8x4 pixels = j40_frame_pixels_u8x4(&frame, J40_RGBA);
        for (int y = 0; y < pixels.height; ++y) {
            //fwrite(j40_row_u8x4(pixels, y), 4, pixels.width, out);
        }
    }

    // J40 stops once the first error is encountered; its error can be checked at the very end
    res = j40_error(&image) ? -1 : 0;

    DEBUG_p("error: %s", j40_error_string(&image));

    j40_free(&image); // also frees all memory associated to j40_frame etc.
    return res;
}


static cairo_surface_t *
_j40_get_image_surface(abydos_plugin_handle_t *h,
                       int page,
                       cairo_bool_t *layer_visibility,
                       int variant,
                       double time,
                       int frame)
{
}


static void
_j40_get_duration(abydos_plugin_handle_t *h, int page, double *duration)
{
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _j40_get_mime_types,
        .new = _j40_new,
        .free = _j40_free,
        .create_from_data = _j40_create_from_data,
        .get_image_surface = _j40_get_image_surface,
        .get_duration = _j40_get_duration
    };
    return &p;
}
