/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <nil-cairo.h>
#include <nil-mcga.h>
#include <stdlib.h>
#include <string.h>


typedef struct{
    cairo_surface_t *surface;
}page_t;

struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    page_t *page;
};


static const char **
_hic_get_mime_types()
{
    static const char *mime_types[] = {
        "image/x-homebrew-icon",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_hic_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    info->threadsafe = 1;
    h->info = info;
    h->page = NULL;
    return h;
}


static void
_hic_free(abydos_plugin_handle_t *h)
{
    if(h->page){
        int i;
        for(i=0;i<h->info->page_count;++i)
            if(h->page[i].surface)
                cairo_surface_destroy(h->page[i].surface);
        free(h->page);
    }
    free(h);
}

/*

identifier led = 18

0x1a 0x1a 0x00 0x00 0x00 0x00

0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00
__=290___ __=19____ __=1012__
0x22 0x01 0x13 0x00 0xf4 0x03 0xff 0xff

0xac 0x14

0x17f8 -> table?

0x2ca4 -> first icon?
0x2da4 -> second icon?

*/

static int
_hic_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
    nile_t ni;
    int i;

    h->info->width = 16;
    h->info->height = 16;

    nile_init(&ni);

    h->info->page_count = (len - 0x2ca4) / 256;

    ni.type = NILE_TYPE_INDEXED;
    ni.width = 16;
    ni.height = 16;
    ni.palette = (uint8_t*)nil_mcga_palette;
    ni.colors = 256;
    ni.format = NILE_FORMAT_r8g8b8;
    ni.bpc = 3;

    h->page = calloc(h->info->page_count, sizeof(page_t));
    for(i=0;i<h->info->page_count;++i){
        ni.pixels = (uint8_t*)data + 0x2ca4 + i * 256;
        h->page[i].surface = nil_cairo_surface_create(&ni, 0);
    }
    nile_done(&ni);
    return 0;
}


static cairo_surface_t *
_hic_get_image_surface(abydos_plugin_handle_t *h,
                       int page,
                       cairo_bool_t *layer_visibility,
                       int variant,
                       double time,
                       int frame)
{
    return cairo_surface_reference(h->page[page].surface);
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _hic_get_mime_types,
        .new = _hic_new,
        .free = _hic_free,
        .create_from_data = _hic_create_from_data,
        .get_image_surface = _hic_get_image_surface
    };
    return &p;
}
