/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <debug.h>
#include <nile.h>
#include <nil-cairo.h>
#include <nil-mcga.h>
#include <stdlib.h>
#include <string.h>


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static const char **
_mgi_get_mime_types()
{
    static const char *mime_types[] = {
        "image/x-micrografx-icon",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_mgi_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    info->threadsafe = 1;
    h->info = info;
    h->surface = NULL;
    return h;
}


static void
_mgi_free(abydos_plugin_handle_t *h)
{
    free(h);
}


/*

__ID_______________ __=365_____________ __=1_______________
0x5a 0x5a 0x5a 0x5a 0x64 0x01 0x00 0x00 0x01 0x00 0x00 0x00 0x00 0x00 0x00 0x00
0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00
                    __entries?_________
0x00 0x00 0x00 0x00 0x48 0x00 0x00 0x00

entries:

     __="view"__________
0x00 0x76 0x69 0x65 0x77 0x00 0x00 0x00 0xe5 0x90 0x08 0x00 0x6e 0x47 0x17 0x00
                         __="view1"______________
0x5c 0x33 0x16 0x00 0x10 0x76 0x69 0x65 0x77 0x31 0x00 0x00 0x18 0x00 0x00 0x00
                                        __=11001100
0x03 0x00 0x00 0x00 0x0c 0x00 0x00 0x00 0xcc

     __="view"_______________
0x00 0x76 0x69 0x65 0x77 0x00 0x00 0x00 0x05 0x90 0x08 0x00 0x5e 0x42 0x17 0x00
                         __="view"__________
0xf8 0x2e 0x16 0x00 0x08 0x76 0x69 0x65 0x77 0x00 0x00 0x00 0x03 0x00 0x00 0x00
                                        __=11100100
0x01 0x00 0x00 0x00 0x04 0x00 0x00 0x00 0xe4

     __="layer"______________
0x00 0x6c 0x61 0x79 0x65 0x72 0x00 0x00 0x05 0x90 0x08 0x00 0x5e 0x42 0x17 0x00
                         __="logos.ged"______________________________
0xf8 0x2e 0x16 0x00 0x2e 0x6c 0x6f 0x67 0x6f 0x73 0x2e 0x67 0x65 0x64 0x00 0x00
                                        __=00111000
0x02 0x00 0x00 0x00 0x08 0x00 0x00 0x00 0x38

     __="layer"______________
0x00 0x6c 0x61 0x79 0x65 0x72 0x00 0x00 0x05 0x90 0x08 0x00 0x5e 0x42 0x17 0x00
                         __="flying.ged"__________________________________
0xf8 0x2e 0x16 0x00 0x2e 0x66 0x6c 0x79 0x69 0x6e 0x67 0x2e 0x67 0x65 0x64 0x00
                                        __=10011000
0x01 0x00 0x00 0x00 0x04 0x00 0x00 0x00 0x98

     __="layer"______________
0x00 0x6c 0x61 0x79 0x65 0x72 0x00 0x00 0x05 0x90 0x08 0x00 0x5e 0x42 0x17 0x00
                         __="lattice.ged"______________________________________
0xf8 0x2e 0x16 0x00 0x2e 0x6c 0x61 0x74 0x74 0x69 0x63 0x65 0x2e 0x67 0x65 0x64
                                        __=11110000
0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0xf0


*/

static int
_mgi_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
    const char *end;
    int count;
    nile_t ni;
    int i;

    count = data[36];
    DEBUG_p("entries: %d", count);
    end = data + len;
    data += 40;

    nile_init(&ni);

    ni.type = NILE_TYPE_INDEXED;
    ni.width = 80;
    ni.height = count * 60;
    //ni.pixels = data + 81 + (80*64+41)*4;
    ni.colors = 256;
    ni.bpc = 3;
    ni.format = NILE_FORMAT_r8g8b8;
    //ni.palette = data + 66 + 16;
    ni.palette = nil_mcga_palette;

    nile_ensure_indexed(&ni, 80, 60 * count);
    for(i=0;i<count;++i){
        memcpy(ni.pixels + 80*60*i, data + 41, 80*60);
        data += 41 + 80*60;
    }

    h->info->width = ni.width;
    h->info->height = ni.height;
    h->surface = nil_cairo_surface_create(&ni, NILE_FLIPPED);

    nile_done(&ni);
    return 0;
}


static cairo_surface_t *
_mgi_get_image_surface(abydos_plugin_handle_t *h,
                       int page,
                       cairo_bool_t *layer_visibility,
                       int variant,
                       double time,
                       int frame)
{
    return cairo_surface_reference(h->surface);
}


static void
_mgi_get_duration(abydos_plugin_handle_t *h, int page, double *duration)
{
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _mgi_get_mime_types,
        .new = _mgi_new,
        .free = _mgi_free,
        .create_from_data = _mgi_create_from_data,
        .get_image_surface = _mgi_get_image_surface,
        .get_duration = _mgi_get_duration
    };
    return &p;
}
