/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef hashmap_h
#define hashmap_h

/* Hash Functions */

typedef unsigned int hash_t;
typedef int(*hashmap_equal_func_t)(const void*,const void*);
typedef hash_t(*hashmap_hash_func_t)(const void*);

hash_t hash_combine(hash_t,hash_t);
int direct_equal(const void*,const void*);
hash_t direct_hash(const void*);
#define pointer_equal direct_equal
hash_t pointer_hash(const void*);
int int_equal(const void*,const void*);
hash_t int_hash(const void*);
int str_equal(const void*,const void*);
hash_t str_hash_update(hash_t,const char* s,int len);
hash_t str_hash(const void*);
int strcase_equal(const void*,const void*);
hash_t strcase_hash_update(hash_t,const char* s,int len);
hash_t strcase_hash(const void*);

/* Hashmap */

typedef struct{
    void *key;
    void *value;
    hash_t hash;
}hashmap_node_t;

typedef struct _hashmap_t hashmap_t;
typedef void(*hashmap_notify_func_t)(void*);
typedef int(*hashmap_foreach_func_t)(void* key,void* value,void* userdata);

struct _hashmap_t{
    /* Size of the allocated memory (in number of items). */
    int size;
    /* Number of items in the hashmap. */
    int nnodes;
    int noccupied;                      /* nnodes + tombstones */
    hashmap_node_t *nodes;
    int mod;
    unsigned int mask;
    hashmap_hash_func_t hash_func;
    hashmap_equal_func_t equal_func;
    hashmap_notify_func_t key_notify_func;
    hashmap_notify_func_t value_notify_func;
};

hashmap_t *hashmap_init(hashmap_t*,hashmap_hash_func_t,hashmap_equal_func_t,hashmap_notify_func_t key_notify,hashmap_notify_func_t value_notify);
void hashmap_done(hashmap_t*);

void  hashmap_add(hashmap_t*,void* key,void* value);
void* hashmap_get(hashmap_t*,const void* key);
const void* hashmap_get_key(hashmap_t*,const void* key);
void* hashmap_get_any(hashmap_t*);
int hashmap_remove(hashmap_t*,const void* key);
void hashmap_remove_all(hashmap_t*);
int hashmap_size(hashmap_t*);
#define hashmap_size(hm) ((hm)->nnodes)
int hashmap_foreach(hashmap_t*,hashmap_foreach_func_t,void*);

#endif /* hashmap_h */
